/*
 *   MIRACL arithmetic routines 0 - Add and subtract routines 
 *   bnarth0.c
 */

#include "miracl.h"

/* Access global variables */

extern small base;   /* number base      */  
extern int nib;      /* length of bigs   */
extern int depth;    /* error tracing .. */
extern int trace[];  /* ... mechanism    */
extern big w0;       /* workspace big    */
extern bool check;   /* overflow check   */

void padd(x,y,z)
big x;
big y;
big z;
{ /*  add two  big numbers, z=x+y where   *
   *  x and y are positive and  x>=y      */
    int lx,ly,i;
    small carry,psum;
    if (ERNUM) return;
    depth++;
    trace[depth]=3;
    if (TRACER) track();
    lx = x[0];
    ly = y[0];
    if (y!=z) copy(x,z);
    else ly=lx;
    carry=0;
    z[0]=lx;
    if (lx<nib || (!check && lx<2*nib)) z[0]++;
    for (i=1;i<=ly || carry>0;i++)
    { /* add by columns */
        if (i>nib && (check || i>2*nib))
        {
            berror(3);
            depth--;
            return;
        }
        psum=x[i]+y[i]+carry;
        carry=0;
        if (psum>=base)
        { /* set carry */
            carry=1;
            psum-=base;
        }
        z[i]=psum;
    }
    lzero(z);
    depth--;
}

void psub(x,y,z)
big x;
big y;
big z;
{  /*  subtract two big numbers z=x-y      *
    *  where x and y are positive and x>y  */
    int lx,ly,i;
    small borrow,pdiff;
    if (ERNUM) return;
    depth++;
    trace[depth]=4;
    if (TRACER) track();
    lx = x[0];
    ly = y[0];
    if (ly>lx)
    {
        berror(4);
        depth--;
        return;
    }
    if (y!=z) copy(x,z);
    else ly=lx;
    z[0]=lx;
    borrow=0;
    for (i=1;i<=ly || borrow>0;i++)
    { /* subtract by columns */
        if (i>lx)
        {
            berror(4);
            depth--;
            return;
        }
        pdiff=x[i]-y[i]-borrow;
        borrow=0;
        if (pdiff<0)
        { /* set borrow */
            pdiff+=base;
            borrow=1;
        }
        z[i]=pdiff;
    }
    lzero(z);
    depth--;
}

void select(x,d,y,z)
big x;
int d;
big y;
big z;
{ /* perform required add or subtract operation */
    int sx,sy,sz,jf,xgty;
    if (ERNUM) return;
    depth++;
    trace[depth]=29;
    if (TRACER) track();
    if (notint(x) || notint(y))
    {
        berror(12);
        depth--;
        return;
    }
    sx=exsign(x);
    sy=exsign(y);
    x[0]=abs(x[0]);  /* force operands to be positive */
    y[0]=abs(y[0]);
    xgty=compare(x,y);
    jf=(1+sx)+(1+d*sy)/2;
    switch (jf)
    { /* branch according to signs of operands */
    case 0:
        if (xgty>=0)
            padd(x,y,z);
        else
            padd(y,x,z);
        sz=MINUS;
        break;
    case 1:
        if (xgty<=0)
        {
            psub(y,x,z);
            sz=PLUS;
        }
        else
        {
            psub(x,y,z);
            sz=MINUS;
        }
        break;
    case 2:
        if (xgty>=0)
        {
            psub(x,y,z);
            sz=PLUS;
        }
        else
        {
            psub(y,x,z);
            sz=MINUS;
        }
        break;
    case 3:
        if (xgty>=0)
            padd(x,y,z);
        else
            padd(y,x,z);
        sz=PLUS;
        break;
    }
    insign(sx,x);  /* restore signs to operands  */
    insign(sy,y);
    insign(sz,z);  /* ... and set sign of result */
    depth--;
}

void add(x,y,z)
big x;
big y;
big z;
{  /* add two signed big numbers together z=x+y */
    if (ERNUM) return;
    depth++;
    trace[depth]=27;
    if (TRACER) track();
    select(x,PLUS,y,z);
    depth--;
}

void subtract(x,y,z)
big x;
big y;
big z;
{ /* subtract two big signed numbers z=x-y */
    if (ERNUM) return;
    depth++;
    trace[depth]=28;
    if (TRACER) track();
    select(x,MINUS,y,z);
    depth--;
}

void incr(x,n,z)
big x;
big z;
small n;
{  /* add int to big number: z=x+n */
    if (ERNUM) return;
    depth++;
    trace[depth]=7;
    if (TRACER) track();
    convert(n,w0);
    select(x,PLUS,w0,z);
    depth--;
}

void decr(x,n,z)
big x;
big z;
small n;
{  /* subtract int from big number: z=x-n */   
    if (ERNUM) return;
    depth++;
    trace[depth]=8;
    if (TRACER) track();
    convert(n,w0);
    select(x,MINUS,w0,z);
    depth--;
}
