/*
 *   MIRACL arithmetic routines 1 - multiplying and dividing BIG NUMBERS by  
 *   integer numbers.
 *   bnarth1.c
 */

#include "miracl.h"

/* access global variables */

extern small base;   /* number base     */
extern int nib;      /* length of bigs  */
extern int depth;    /* error tracing ..*/
extern int trace[];  /* .. mechanism    */
extern bool check;   /* overflow check  */

void premult(x,n,z)
big x;
big z;
small n;
{ /* premultiply a big number by an int z=x.n */
    int s,xl,m;
    small carry;
    if (ERNUM) return;
    depth++;
    trace[depth]=9;
    if (TRACER) track();
    if (notint(x))
    {
        berror(12);
        depth--;
        return;
    }
    s=exsign(x)*sign(n);  /* calculate sign of result */
    n=abs(n);
    if (n==0)  /* test for some special cases  */
    {
        zero(z);
        depth--;
        return;
    }
    if (n==1)
    {
        copy(x,z);
        insign(s,z);
        depth--;
        return;
    }
    if (n==2)
    {
        add(x,x,z);
        insign(s,z);
        depth--;
        return;
    }
    xl=abs(x[0]);
    if (x!=z) zero(z);
    m=0;
    carry=0;
    while (m<xl || carry>0)
    { /* multiply each digit of x by n */
        m++;
        if (m>nib && (check || m>2*nib))
        {
            berror(3);
            depth--;
            return;
        }
        carry=muldiv(x[m],n,carry,base,&z[m]);
        z[0]=m;
    }
    insign(s,z);
    depth--;
}
         
small subdiv(x,n,z)
big x;
big z;
small n;
{  /*  subdivide a big number by an int   z=x/n  *
    *  returns int remainder                     */ 
    int i,s,sx,xl;
    small r;
    if (ERNUM) return 0;
    depth++;
    trace[depth]=10;
    if (TRACER) track();
    if (notint(x)) berror(12);
    r=0;
    if (n==0) berror(2);
    if (ERNUM) 
    {
        depth--;
        return 0;
    }
    sx=exsign(x);
    s=sx*sign(n);  /* set sign of result */
    n=abs(n);
    if (n==1) /* special case */
    {
        copy(x,z);
        insign(s,z);
        depth--;
        return r;
    }
    xl=abs(x[0]);
    if (x!=z) zero(z);
    for (i=xl;i>0;i--)
    { /* divide each digit of x by n */
        z[i]=muldiv(r,base,x[i],n,&r);
    }
    z[0]=xl;
    lzero(z);
    insign(s,z);
    depth--;
    return r*sx;
}
