/*
 *   MIRACL floating-Slash arithmetic
 *   bnflash.c
 */

#include "miracl.h"

/* Access global variables */

extern int  depth;    /* error tracing ..*/
extern int  trace[];  /* .. mechanism    */
extern bool check;    /* overflow check  */
extern int  nib;      /* length of bigs  */

extern big w1;        /* workspace variables  */
extern big w2;
extern big w3;
extern big w4;
extern big w5;
extern big w6;
extern big w0;

void flop(x,y,op,z)
flash x,y,z;
int op;
{ /* Do basic flash operations */
    if (ERNUM) return;
    depth++;
    trace[depth]=69;
    if (TRACER) track();
    numer(x,w1);
    denom(x,w2);
    numer(y,w3);
    denom(y,w4);
    check=OFF;
    switch (op)
    {
    case 0 : /* multiply */
            if (x==y)
            {
                multiply(w1,w1,w5);
                multiply(w2,w2,w6);
            }
            else
            {
                multiply(w1,w3,w5);
                multiply(w2,w4,w6);
            }
            break;
    case 1 : /* divide   */
            multiply(w1,w4,w5);
            multiply(w2,w3,w6);
            break;
    case 2 : /* add      */
            multiply(w1,w4,w5);
            multiply(w2,w4,w6);
            multiply(w2,w3,w0);
            add(w5,w0,w5);
            break;
    case 3 : /* subtract */
            multiply(w1,w4,w5);
            multiply(w2,w4,w6);
            multiply(w2,w3,w0);
            subtract(w5,w0,w5);
            break;
    case 4 : /* (x+y)/(x-y)     */
            multiply(w1,w4,w5);
            multiply(w2,w3,w0);
            subtract(w5,w0,w6);
            add(w5,w0,w5);
            break;
    case 5 : /* (x+y)/(1-x.y)   */
            multiply(w1,w4,w5);
            multiply(w2,w3,w0);
            add(w5,w0,w5);
            if (x==y)
            {
                multiply(w2,w2,w6);
                multiply(w1,w1,w0);
            }
            else
            {
                multiply(w2,w4,w6);
                multiply(w1,w3,w0);
            }
            subtract(w6,w0,w6);
            break;
    case 6 : /* (x.y-1)/(x.y+1) */
            if (x==y)
            {
                multiply(w1,w1,w5);
                multiply(w2,w2,w0);
            }
            else
            {
                multiply(w1,w3,w5);
                multiply(w2,w4,w0);
            }
            add(w5,w0,w6);
            subtract(w5,w0,w5);
            break;
    case 7 : /* (x+y)/(1+x.y)   */
            multiply(w1,w4,w5);
            multiply(w2,w3,w0);
            add(w5,w0,w5);
            if (x==y)
            {
                multiply(w2,w2,w6);
                multiply(w1,w1,w0);
            }
            else
            {
                multiply(w2,w4,w6);
                multiply(w1,w3,w0);
            }
            add(w6,w0,w6);
            break;
    }
    check=ON;
    round(w5,w6,z);
    depth--;
}

void fmul(x,y,z)
flash x,y,z;
{ /* Flash multiplication - z=x*y */
    if (ERNUM) return;
    depth++;
    trace[depth]=35;
    if (TRACER) track();
    flop(x,y,0,z);
    depth--;
}

void fdiv(x,y,z)
flash x,y,z;
{ /* Flash divide - z=x/y */
    if (ERNUM) return;
    depth++;
    trace[depth]=36;
    if (TRACER) track();
    flop(x,y,1,z);
    depth--;
}

void fadd(x,y,z)
flash x,y,z;
{ /* Flash add - z=x+y */
    if (ERNUM) return;
    depth++;
    trace[depth]=37;
    if (TRACER) track();
    flop(x,y,2,z);
    depth--;
}

void fsub(x,y,z)
flash x,y,z;
{ /* Flash subtract - z=x-y */
    if (ERNUM) return;
    depth++;
    trace[depth]=38;
    if (TRACER) track();
    flop(x,y,3,z);
    depth--;
}

int fcomp(x,y)
flash x,y;
{ /* compares two Flash numbers             *
   * returns -1 if y>x; +1 if x>y; 0 if x=y */
    if (ERNUM) return 0;
    depth++;
    trace[depth]=39;
    if (TRACER) track();
    numer(x,w1);
    denom(y,w2);
    check=OFF;
    multiply(w1,w2,w5);
    numer(y,w1);
    denom(x,w2);
    multiply(w1,w2,w0);
    check=ON;
    depth--;
    return (compare(w5,w0));
}

void ftrunc(x,y,z)
flash x,z;
big y;
{ /* sets y=int(x), z=rem(x) - returns *
   * y only for ftrunc(x,y,y)          */
    if (ERNUM) return;
    depth++;
    trace[depth]=45;
    if (TRACER) track();
    numer(x,w1);
    denom(x,w2);
    divide(w1,w2,w3);
    copy(w3,y);
    if (y!=z) fpack(w1,w2,z);
    depth--;
}

void fconv(n,d,x)
small n,d;
flash x;
{ /* convert simple fraction n/d to Flash form */
    small g;
    if (ERNUM) return;
    depth++;
    trace[depth]=40;
    if (TRACER) track();
    if (d<0)
    {
        d=(-d);
        n=(-n);
    }
    g=igcd(n,d);
    n/=g;
    d/=g;
    convert(n,w5);
    convert(d,w6);
    fpack(w5,w6,x);
    depth--;
}

void frecip(x,y)
flash x,y;
{ /* set y=1/x */
    if (ERNUM) return;
    depth++;
    trace[depth]=41;
    if (TRACER) track();
    numer(x,w1);
    denom(x,w2);
    fpack(w2,w1,y);
    depth--;
}

void fpmul(x,n,d,y)
flash x,y;
small n,d;
{ /* multiply x by small fraction n/d - y=x*n/d */
    small r,g;
    if (ERNUM) return;
    if (n==0 || size(x)==0)
    {
        zero(y);
        return;
    }
    if (n==d)
    {
        copy(x,y);
        return;
    }
    depth++;
    trace[depth]=42;
    if (TRACER) track();
    if (d<0)
    {
        d=(-d);
        n=(-n);
    }
    g=igcd(n,d);
    n/=g;
    d/=g;
    numer(x,w1);
    denom(x,w2);
    r=subdiv(w1,d,w3);
    g=igcd(d,r);
    r=subdiv(w2,n,w3);
    g*=igcd(n,r);
    check=OFF;
    premult(w1,n,w5);
    premult(w2,d,w6);
    subdiv(w5,g,w5);
    subdiv(w6,g,w6);
    check=ON;
    if (abs(w5[0])+abs(w6[0])<=nib)
        fpack(w5,w6,y);
    else
        round(w5,w6,y);
    depth--;
}

void fincr(x,n,d,y)
flash x,y;
small n,d;
{ /* increment x by small fraction n/d - y=x+(n/d) */
    if (ERNUM) return;
    depth++;
    trace[depth]=43;
    if (TRACER) track();
    if (d<0)
    {
        d=(-d);
        n=(-n);
    }
    numer(x,w1);
    denom(x,w2);
    check=OFF;
    premult(w1,d,w5);
    premult(w2,d,w6);
    premult(w2,n,w0);
    add(w5,w0,w5);
    check=ON;
    if (d==1 && (abs(w5[0])+abs(w6[0])<=nib))
        fpack(w5,w6,y);
    else
        round(w5,w6,y);
    depth--;
}
