/*
 *   MIRACL Extended Greatest Common Divisor module.
 *   bnxgcd.c
 */

#include "miracl.h"

/* Access global variables */

extern int   depth;    /* error tracing ..*/
extern int   trace[];  /* .. mechanism    */
extern small base;     /* number base     */

extern big w1;       /* workspace variables  */
extern big w2;
extern big w3;
extern big w4;
extern big w0;

void xgcd(x,y,xd,yd,z)
big x,y,xd,yd,z;
{ /* greatest common divisor by Euclids method  *
   * extended to also calculate xd and yd where *
   *      z = x.xd + y.yd = gcd(x,y)            */
    small u,v,q,r,a,b,c,d;
    int n;
    big t;
    if (ERNUM) return;
    depth++;
    trace[depth]=30;
    if (TRACER) track();
    if (xd==yd || xd==z || yd==z)
    {
        berror(7);
        depth--;
        return;
    }
    copy(x,w1);
    copy(y,w2);
    insign(PLUS,w1);
    insign(PLUS,w2);
    copy(w1,w3);
    copy(w2,w4);
    convert((small)1,xd);
    zero(yd);
    a=b=c=d=0;
    while (size(w2)!=0)
    {
        if (b==0)
        { /* update w1 and w2 */
            divide(w1,w2,z);
            t=w1,w1=w2,w2=t;    /* swap(w1,w2) */
            multiply(yd,z,w0);
            subtract(xd,w0,xd);
            t=xd,xd=yd,yd=t;    /* swap(xd,yd) */
        }
        else
        {
            premult(w1,c,z);
            premult(w1,a,w1);
            premult(w2,b,w0);
            premult(w2,d,w2);
            add(w1,w0,w1);
            add(w2,z,w2);
            premult(xd,c,z);
            premult(xd,a,xd);
            premult(yd,b,w0);
            premult(yd,d,yd);
            add(xd,w0,xd);
            add(yd,z,yd);
        }
        if (ERNUM || size(w2)==0) break;
        n=w1[0];
        a=1;
        b=0;
        c=0;
        d=1;
        if (n==1)
        {
            u=w1[1];
            v=w2[1];
        }
        else
        {
            u=muldiv(w1[n],base,w1[n-1],w1[n]+1,&r);
            v=muldiv(w2[n],base,w2[n-1],w1[n]+1,&r);
        }
        forever
        { /* work only with most significant piece */
            if (n==1)
            {
                if (v==0) break;
                q=u/v;
            }
            else
            {
                if (((v+c)==0) || ((v+d)==0)) break;
                q=(u+a)/(v+c);
                if (q!=(u+b)/(v+d)) break;
            }
            r=a-q*c;
            a=c;
            c=r;
            r=b-q*d;
            b=d;
            d=r;
            r=u-q*v;
            u=v;
            v=r;
        }
    }
    negate(w3,w3);
    mad(w3,xd,w1,w4,yd,yd);
    copy(w1,z);
    depth--;
}
