/*
 *   Program to factor big numbers using Brillhart-Morrison  
 *   method with Pomerance/Wagstaff early abort technique.
 *
 */

#include <stdio.h>
#include <math.h> 
#include "miracl.h"

#define MEM 300        /* Maximum size of factor base   */
#define MLF 100        /* Max. number of large factors  */
#define PAK 1+MEM/(2*BTS)
#define LOG log10      /* if log10 not available use any other log */

small spmd(x,n,m)
small x,n,m;
{ /*  returns x^n mod m  */
    small r;
    r=0;
    if (x==0) return r;
    r=1;
    if (n==0) return r;
    forever
    {
        if (n%2!=0) muldiv(r,x,0,m,&r);
        n/=2;
        if (n==0) return r;
        muldiv(x,x,0,m,&x);
    }
}

int knuth(ep,N,R,D,T)
int ep[];
big N,R,D,T;
{ /* initialization */
    double fks,dp,top;
    bool found;
    int i,j,mm,bk,nk,m,d,kk;
    static int K[]={0,1,2,3,5,6,7,10,11,13,14,15,17,0};
    small r,p;
    char ch;
    printf("input number to be factored N= \n");
    d=cinnum(N,stdin);
    if (prime(N))
    {
        printf("This number is prime!\n");
        exit(0);
    }
    if (d<10) m=d;
    else m=25;
    if (d>20) m=30*(d-20);
    if (m>MEM) m=MEM;
    top=(-10.0e0);
    found=FALSE;
    nk=0;
    bk=0;
    ep[0]=1;
    ep[1]=2;
    do
    { /* search for best Knuth-Schroepel multiplier */
        mm=m;
        kk=K[++nk];
        if (kk==(-1)) continue;
        if (kk==0)
        {
            kk=K[bk];
            if (kk==(-1)) exit(0);
            printf("recommended k=%d\n",kk);
            printf("    accepted? ");
            scanf("%c",&ch);
            getchar();
            if (ch=='Y' || ch== 'y') found=TRUE;
            else
            { /* strike out this one and try again */
                top=(-10.0e0);
                K[bk]=(-1);
                nk=0;
                continue;
            }
        }
        premult(N,kk,D);
        if (root(D,2,R))
        {
            printf("%dN is a perfect square!\n",kk);
            K[nk]=(-1);
            continue;
        }
        fks=LOG(2.0e0)/(3.0e0);
        r=subdiv(D,8,T);
        if (r==1) fks*=(4.0e0);
        if (r==5) fks*=(2.0e0);
        fks-=LOG((double)kk)/(2.0e0);
        i=0;
        j=1;
        while (j<m)
        { /* select small primes */
            p=PRIMES[++i];
            if (p==0)
            {
                mm=j;
                break;
            }
            r=subdiv(D,p,T);
            if (spmd(r,(p-1)/2,p)<=1)
            { /* use only if Jacobi symbol = 0 or 1 */
                ep[++j]=p;
                dp=(double)p;
                fks+=LOG(dp)/(dp+1.0e0);
                if (kk%p!=0) fks+=LOG(dp)/(dp-1.0e0);
            }
        }
        if (fks>top)
        { /* find biggest fks */
            top=fks;
            bk=nk;
        }
    } while (!found);
    printf("using multiplier k= %d\n",kk);
    printf("and %d small primes as factor base\n",mm);
    return mm;
}

main()
{ /* factoring via continued fractions */
    big N,T,D,R,RD,U,UD,V,VD,P,PD,A,X,Y,B[4];
    static big x[MEM+1],y[MEM+1],z[MLF+1],w[MLF+1];
    static small epp[MEM+1];
    static int ep[MEM+1],pr[MLF+1];
    static int b[MEM+1],e[MEM+1];
    static unsigned int E[MEM+1][PAK],G[MLF+1][PAK];
    static int ma[4]={0,10,50,150};
    static int dv[4]={1,500,600,1000};
    int S,mm,i,j,k,n,rb,jj,nbts,nlp,st,r,lp;
    small sr;
    bool abort,found,facted,partial;
    mirsys(-20,MAXBASE);
    N=mirvar(0);   /* initialize all big numbers */
    T=mirvar(0);
    D=mirvar(0);
    R=mirvar(0);
    RD=mirvar(0);
    U=mirvar(0);
    UD=mirvar(0);
    V=mirvar(0);
    VD=mirvar(0);
    P=mirvar(0);
    PD=mirvar(0);
    A=mirvar(0);
    X=mirvar(0);
    Y=mirvar(0);
    for (i=0;i<=3;i++)
        B[i]=mirvar(0);
    for (i=0;i<=MEM;i++)
    {
        x[i]=mirvar(0);
        y[i]=mirvar(0);
    }
    for (i=0;i<=MLF;i++)
    {
        z[i]=mirvar(0);
        w[i]=mirvar(0);
    }
    mm=knuth(ep,N,R,D,T);
    st=1;
    sr=1;
    for (j=1;j<=mm;j++)
    {
        epp[j]=0;
        if (ep[j]>MAXBASE/sr)
        {
            epp[st]=sr;
            st=j;
            sr=1;
        }
        sr*=ep[j];
    }
    epp[st]=sr;
    premult(R,2,RD);
    copy(RD,UD);
    copy(UD,U);
    convert(1,V);
    multiply(R,R,T);
    subtract(D,T,VD);
    copy(R,P);
    convert(1,PD);
    zero(A);
    nbts=8*sizeof(int);
    for (i=0;i<=mm;i++)
    { 
        b[i]=(-1);
        for (j=0;j<=mm/nbts;j++)
            E[i][j]=0;
    }
    for (i=0;i<=MLF;i++)
        for (j=0;j<=mm/nbts;j++)
            G[i][j]=0;
    S=0;
    jj=0;
    nlp=0;
    copy(RD,B[0]);
    for (i=1;i<=3;i++) /* set early aborts */
        subdiv(B[i-1],dv[i],B[i]);
    printf("starting...\n");
    forever
    { /* main loop - generating continuous *
       * fraction cycle for sqrt(kN)       */
        copy(V,T);
        subtract(UD,U,R);
        multiply(A,R,V);
        add(VD,V,V);
        copy(T,VD);
        copy(U,UD);
        divide(U,V,A);
        subtract(RD,U,U);
        S=1-S;
        e[0]=S;
        for (i=1;i<=mm;i++)
            e[i]=0;
        copy(V,T);
        partial=FALSE;
        facted=FALSE;
        for (j=1;j<=mm;j++)
        { /* now attempt complete factorisation of T */
            abort=FALSE;
            for (i=1;i<=3;i++) /* if not making good progress ... */
                if (j==ma[i] && compare(T,B[i])>0) abort=TRUE;
            if (abort) break;  /* ... abort */
            if (epp[j]>0) sr=subdiv(T,epp[j],X);
            if ((sr%ep[j])!=0) continue;
            if (epp[j]==ep[j])
            {
                e[j]++;
                copy(X,T);
            }
            while (subdiv(T,ep[j],X)==0)
            { /* cast out ep[j] */
                e[j]++;
                copy(X,T);
            }
            st=size(T);
            if (st==1)
            {
               facted=TRUE;
               break;
            }
            if (size(X)<=ep[j])
            { /* st is prime < ep[mm]^2 */
                if (st==TOOBIG || st>3*ep[mm]) break;
                if (st<=ep[mm])
                    for (i=j;i<=mm;i++)
                    if (st==ep[i])
                    {
                        e[i]++;
                        facted=TRUE;
                        break;
                    }
                if (facted) break;
                lp=st;  /* factored with large prime */
                partial=TRUE;
                facted=TRUE;
                break;
            }
        }
        found=FALSE;
        if (facted)
        { /* T factorised completely */
            copy(P,X);
            convert(1,Y);
            for (k=1;k<=mm;k++)
            { /* build up square part in Y  *
               * reducing e[k] to 0s and 1s */
                if (e[k]<2) continue;
                r=e[k]/2;
                e[k]%=2;
                convert(ep[k],T);
                power(T,r,T);
                multiply(T,Y,Y);
            }
/* debug only - print out details of each factorization  
            cotnum(X,stdout);
            cotnum(Y,stdout);
            if (e[0]==1) printf("-1");
            else printf("1");
            for (k=1;k<=mm;k++)
            {
                if (e[k]==0) continue;
                printf(".%d",ep[k]);
            }
            if (partial) printf(".%d\n",lp);
            else printf("\n");
*/
            found=TRUE;
            if (partial)
            { /* factored with large prime */
                found=FALSE;
                for (i=0;i<nlp;i++)
                { /* search for matching large prime */
                    if (lp!=pr[i]) continue;
                    found=TRUE;
                    break;
                }
                if (!found && nlp<MLF)
                { /* store new partial factorization */
                    pr[nlp]=lp;
                    copy(X,z[nlp]);
                    copy(Y,w[nlp]);
                    for (n=0,rb=0,j=0;j<=mm;j++)
                    {
                        G[nlp][n]|=((e[j]&1)<<rb);
                        if (++rb==nbts) n++,rb=0;
                    }
                    nlp++;
                }
                if (found)
                { /* match found so use as factorization */
                    printf("*");
                    mad(X,z[i],X,N,N,X);
                    mad(Y,w[i],Y,N,N,Y);
                    for (n=0,rb=0,j=0;j<=mm;j++)
                    {
                        e[j]+=((G[i][n]>>rb)&1);
                        if (e[j]==2)
                        {
                            premult(Y,ep[j],Y);
                            divide(Y,N,N);
                            e[j]=0;
                        }
                        if (++rb==nbts) n++,rb=0;
                    }
                    premult(Y,lp,Y);
                    divide(Y,N,N);
                }
            }
            else printf(" ");
            if (found) for (k=mm;k>=0;k--)
            { /* use new factorization in search for solution */
                if (e[k]==0) continue;
                if (b[k]<0)
                { /* no solution this time - store   *
                   * details in E, x and y for later */
                    b[k]=jj;
                    copy(X,x[jj]);
                    copy(Y,y[jj]);
                    for (n=0,rb=0,j=0;j<=mm;j++)
                    {
                        E[jj][n]|=((e[j]&1)<<rb);
                        if (++rb==nbts) n++,rb=0;
                    }
                    jj++;
                    printf("working...%d\n",jj);
                    found=FALSE;
                    break;
                }
                i=b[k];
                mad(X,x[i],X,N,N,X);
                mad(Y,y[i],Y,N,N,Y);
                for (n=0,rb=0,j=0;j<=mm;j++)
                { /* Gaussian elimination */
                    e[j]+=((E[i][n]>>rb)&1);
                    if (e[j]==2)
                    {
                        premult(Y,ep[j],Y);
                        divide(Y,N,N);
                        e[j]=0;
                    }
                    if (++rb==nbts) n++,rb=0;
                }
            }
            if (found)
            { /* check for false alarm */
                printf("trying...\n");
                add(X,Y,T);
                if (compare(X,Y)==0 || compare(T,N)==0) found=FALSE;
            }
            if (found)
            { /* factors found! */
                gcd(T,N,P);
                printf("factors found\n");
                cotnum(P,stdout);
                divide(N,P,N);
                cotnum(N,stdout);
                break;
            }
        }
        if (size(V)==1)
        { /* end of continued fraction cycle */
            printf("no factors found\ntry another value of k\n");
            break;
        }
        copy(P,T);
        mad(A,P,PD,N,N,P);
        copy(T,PD);
    }
}
