/*
 *   main MIRACL header - miracl.h.
 */

#include "mirdef.h"

extern bool WRAP;    /* =ON for wraparound on output, else =OFF      */
extern bool ERCON;   /* error control, TRUE for user processing      */
extern int  ERNUM;   /* last recorded error       */
extern bool STROUT;  /* redirect output to OBUFF  */
extern int  IOBASE;  /* base for I/O       */
extern int *PRIMES;  /* Prime number array */
extern int  LINE;    /* Output line length */ 
extern char OBUFF[]; /* Output buffer      */
extern char IBUFF[]; /* Input buffer       */
extern bool POINT;   /* =ON for radix point, =OFF for fractions      */
extern bool EXACT;   /* exact flag         */
extern int  NTRY;    /* no. of tries for prob. primality testing     */
extern bool TRACER;  /* turns track tracer on/off */

/* Function definition  */

/* Group 0 - Internal routines */

extern void  berror();         /* MIRACL error routine          */
extern void  setbase();        /* set internal number base      */
extern void  track();          /* routine tracker               */
extern void  lzero();          /* suppress leading zeros        */
extern bool  notint();         /* tests for flash or big        */
extern int   lent();           /* length of big/flash           */
extern void  padd();           /* primitive add                 */
extern void  psub();           /* primitive subtract            */
extern void  select();         /* selects padd or psub          */
extern small muldiv();         /* int multiply, add and divide  */
extern small igcd();           /* integer GCD                   */
extern int   isqrt();          /* integer square root           */
extern void  shift();          /* shift left or right           */ 

/* Group 1 - General purpose, I/O and basic arithmetic routines  */

extern void  irand();          /* initialize random numbers      */
extern small brand();          /* integer random numbers         */
extern void  zero();           /* set flash number to zero       */
extern void  convert();        /* convert int number to big      */
extern flash mirvar();         /* initialize big/flash number    */
extern void  mirsys();         /* initialize system              */
extern int   exsign();         /* extract sign of flash number   */
extern void  insign();         /* assert sign of flash number    */
extern int   getdig();         /* extract digit from big number  */
extern int   numdig();         /* return number of digits in big */
extern void  putdig();         /* insert digit into big number   */
extern void  copy();           /* copy one flash to another      */
extern void  negate();         /* negate flash number            */
extern void  absol();          /* absolute value of flash number */
extern int   size();           /* convert big number to int      */
extern void  fpack();          /* make flash number from bigs    */
extern void  numer();          /* extract flash numerator        */
extern void  denom();          /* extract flash denominator      */
extern int   compare();        /* compare two big numbers        */
extern void  add();            /* add two big numbers            */
extern void  subtract();       /* subtract two big numbers       */
extern void  incr();           /* add int number to big          */        
extern void  decr();           /* subtract int number from big   */
extern void  premult();        /* multiply big number by int     */
extern small subdiv();         /* divide big number by int       */
extern void  multiply();       /* multiply two big numbers       */
extern void  divide();         /* divide big number by another   */
extern void  mad();            /* multiply, add and divide       */
extern void  build();          /* build flash from c.f generator */
extern void  round();          /* make rounded flash from 2 bigs */
extern int   innum();          /* input  flash number            */
extern void  otnum();          /* output flash number            */
extern int   cinnum();         /* input  flash number - any base */
extern void  cotnum();         /* output flash number - any base */

/* Group 2 - Advanced arithmetic routines */

extern void  gcd();            /* find GCD of two big numbers    */
extern void  xgcd();           /* extended GCD routine           */
extern int   logb2();          /* returns no. of bits in a big   */
extern void  expb2();          /* raises 2 to int power          */
extern void  power();          /* raise big to int power         */
extern void  powmod();         /* raise big to big power mod big */
extern bool  root();           /* find root of big number        */
extern void  bigrand();        /* generate big random number     */
extern void  bigdig();         /* find random big of fixed size  */
extern bool  prime();          /* test big number for primality  */
extern void  nxprime();        /* find next big prime number     */
extern void  strongp();        /* generate 'strong' prime        */

/* Group 3 - Floating-slash routines      */

extern void  flop();           /* primitive flash operations     */
extern void  fmul();           /* multiply flash nos.            */
extern void  fdiv();           /* divide flash numbers           */
extern void  fadd();           /* add flash numbers              */
extern void  fsub();           /* subtract flash nos.            */
extern int   fcomp();          /* compare flash numbers          */
extern void  fconv();          /* convert fraction to flash      */
extern void  frecip();         /* flash reciprical               */
extern void  ftrunc();         /* truncate flash number          */
extern void  fpmul();          /* multiply flash by fraction     */
extern void  fincr();          /* increment flash by fraction    */
extern void  dconv();          /* convert double to flash        */
extern double fdsize();        /* convert flash to double        */
extern void  frand();          /* generates flash random 0<x<1   */

/* Group 4 - Advanced Flash routines */ 

extern void  fpower();         /* raises flash to integer power  */
extern bool  froot();          /* find root of flash number      */
extern void  fpi();            /* returns pi by Guass-Legendre   */
extern void  fexp();           /* exponentiate flash number      */
extern void  flog();           /* log of flash number to base e  */
extern void  fpowf();          /* flash raised to flash power    */    
extern void  ftan();           /* tan of flash number            */
extern void  fatan();          /* arctan of flash number         */
extern void  fsin();           /* sine   of flash number         */
extern void  fasin();          /* arcsin of flash number         */
extern void  fcos();           /* cosine of flash number         */
extern void  facos();          /* arccos of flash number         */
extern void  ftanh();          /* hyperbolic tan of flash number */
extern void  fatanh();         /* hyperbolic arctan              */
extern void  fsinh();          /* hyperbolic sin                 */
extern void  fasinh();         /* hyperbolic arcsin              */
extern void  fcosh();          /* hyperbolic cos                 */
extern void  facosh();         /* hyperbolic arccos              */
