/************************************************************************/
/* RBSLIB Header file                                                   */
/* Copyright 1988 by Robert B. Stout dba MicroFirm                      */
/* All rights reserved                                                  */
/* Compiled by and for use with Zortech C/C++ 1.07                      */
/************************************************************************/

#ifndef DOS_H
#include <dos.h>
#endif /* DOS_H */
#include <stddef.h>

extern unsigned char _osmajor;          /* DOS major version number.    */
extern unsigned char _osminor;          /* DOS minor version number.    */

/************************************************************************/
/* Miscellaneous #defines                                               */
/************************************************************************/

#define Repeat(x) if (x) do
#define Until(x) while(!(x))
#ifndef EVER
#define EVER (;;)       /* e.g. "for EVER {...}"                        */
#endif
#define LAST_CHAR(string) (((char *)string)[strlen(string)-1])
#define NEXT_TO_LAST_CHAR(string) (((char *)string)[strlen(string)-2])
#define STRING_TERMINATOR(string) (((char *)string)[strlen(string)])
#define STREQ(str1,str2) (!strcmp(str1,str2))
#define KB_flush() while(kbhit()) getch()
#define MAX_FLEN 67
#ifndef offsetof
#define offsetof(st,el) (size_t)&(((st *)0)->el)
#endif
#define __PAGE(title)
#define BitSet(arg,posn) (arg | (1 << posn))
#define BitClr(arg,posn) (arg & ~(1 << posn))
#define BitTst(arg,posn) (arg & (1 << posn))

/* The following were present in DLC, but removed from ZTC      */
#define min(x,y) ((x <= y) ? x : y)
#define max(x,y) ((x >= y) ? x : y)

#undef TRUE
#undef FALSE
typedef enum {ERROR = -1, FALSE, TRUE} LOGICAL;

/************************************************************************/
/* The following 2 functions added 1/20/88 by Bob Stout and are         */
/* equivalent to MSC's dos_find_first() and dos_find_next() functions   */
/* NOTE: The operational sequence of these is different from DLC's      */
/*       findfirst(), findnext() functions. These are recursive         */
/*       functions which allows recursively nested routines to use the  */
/*       DOS find functions. rfind_1st() functions similarly to         */
/*       findfirst(), but allows passing a pointer to a previously      */
/*       declared FIND (dta) structure. rfind_nxt() must be called with */
/*       the argument returned by its corresponding rfind_1st() call.   */
/*       Also, when you are "through" using any particular rfind_1st()  */
/*       (i.e. you aren't going to issue any more rfind_next() calls    */
/*       using its argument), you should call free() to deallocate the  */
/*       corresonding FIND structure's memory. rfind_nxt()'s second     */
/*       argument is a "keep" flag to facilitate this. If zero, the     */
/*       FIND structured will be freed if there are no more matches.    */
/*       If non-zero, the structure will not be freed.                  */
/************************************************************************/

struct FIND *rfind_1st(char *, int, struct FIND *),
	    *rfind_nxt(struct FIND *,int);

#ifndef RF_KEEP         /* Tell rfind_nxt() to keep the FIND struct     */
#define RF_KEEP 1
#endif

#ifndef RF_FREE         /* Tell rfind_nxt() to free the FIND struct     */
#define RF_FREE 0
#endif

#define _dos_findfirst(path,attrib,ffblk) (NULL==rfind_1st(path,attrib,ffblk))
#define _dos_findnext(ffblk) 		  (NULL==rfind_nxt(ffblk,RF_KEEP))

/************************************************************************/
/*                                                                      */
/*  Display text with attribute specified                               */
/*                                                                      */
/************************************************************************/

void disp_putsa(char *, int);

/************************************************************************/
/*                                                                      */
/*  BASIC-like string functions                                         */
/*                                                                      */
/************************************************************************/

extern int STR_pool_size, STRALLOC_OK;

char *stralloc(int), *left(char *, int), *right(char *, int),
     *mid(char *, int, int), *string_add(char *, ...);
int  str_init(int, int);
void str_free(void);

/************************************************************************/
/*                                                                      */
/*  Encrypt/decrypt buffer data                                         */
/*                                                                      */
/************************************************************************/

extern char *cryptext;          /* The actual encryption/decryption key */
extern int crypt_ptr;           /* Circular pointer to elements of key  */
extern int crypt_length;        /* Set by caller to strlen(cryptext)    */

void crypt(char *);

/************************************************************************/
/*                                                                      */
/*  Normalize file names                                                */
/*                                                                      */
/************************************************************************/

int flnorm(char *, char *);
char *flscrunch(char *);

/************************************************************************/
/*                                                                      */
/*  Get/set file attributes                                             */
/*                                                                      */
/************************************************************************/

int flsetatr(char *, int);
int flgtatr(char *);

/************************************************************************/
/*                                                                      */
/*  Utility structures for accessing file dates and times               */
/*                                                                      */
/************************************************************************/

struct DOS_TIME
{
        unsigned int ss : 5;
        unsigned int mm : 6;
        unsigned int hh : 5;
} ;
#define dos_time(t) (*(struct DOS_TIME *)(&(t)))
 
struct DOS_DATE
{
        unsigned int da : 5;
        unsigned int mo : 4;
        unsigned int yr : 7;
} ;
#define dos_date(t) (*(struct DOS_DATE *)(&(t)))

#if 0
 
/*  Sample file date and time display.*/
 
#include <stdio.h>
#include <dos.h>
 
main(int argc, char *argv[])
{
	struct FIND *ffblk;
 
        if (2 > argc)
        {
                puts("\aUsage: SHOWDATE filename[.ext]");
                exit(1);
        }
        if (!(ffblk = findfirst(argv[1], 0xff)))
        {
                printf("\aCant find %s\n", argv[1]);
                exit(2);
        }
        printf("File date is %d-%d-%d\n",
                dos_date(ffblk->date).mo,
                dos_date(ffblk->date).da,
                (dos_date(ffblk->date).yr + 80) % 100);
        printf("File time is %d:%d:%d\n",
                dos_time(ffblk->time).hh,
                dos_time(ffblk->time).mm,
                dos_time(ffblk->time).ss);
}
#endif
